package pl.amu.edu.demo.logic;

import lombok.RequiredArgsConstructor;
import pl.amu.edu.demo.data.WeWillStealYourFlatCheckResult;

import static pl.amu.edu.demo.data.WeWillStealYourFlatCheckResult.*;

public interface NetWorthDecider {

    WeWillStealYourFlatCheckResult isEligible(double netWorth);


    @RequiredArgsConstructor(staticName = "forThresholds")
    class ThresholdPersonDecider implements NetWorthDecider {

        final double ineligibilityThreshold;
        final double backofficeDecisionThreshold;

        @Override
        public WeWillStealYourFlatCheckResult isEligible(double netWorth) {
            if (netWorth < ineligibilityThreshold) {
                return NOT_ELIGIBLE;
            }
            if (netWorth < backofficeDecisionThreshold) {
                return BACKOFFICE_DECISION;
            }
            return ELIGIBLE;
        }

    }


    static NetWorthDecider getInstance(boolean isPersonMarried) {
        return isPersonMarried ?
                new ThresholdPersonDecider(500000d, 550000d) :
                new ThresholdPersonDecider(300000d, 350000d);
    }

}
