package pl.amu.edu.demo.logic;

import org.junit.jupiter.api.extension.ExtensionContext;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.ArgumentsProvider;
import pl.amu.edu.demo.data.Address;
import pl.amu.edu.demo.data.Housing;
import pl.amu.edu.demo.data.Person;
import pl.amu.edu.demo.data.WeWillStealYourFlatCheckResult;

import java.time.LocalDate;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import static pl.amu.edu.demo.data.WeWillStealYourFlatCheckResult.ELIGIBLE;
import static pl.amu.edu.demo.data.WeWillStealYourFlatCheckResult.NOT_ELIGIBLE;

@SuppressWarnings("SpellCheckingInspection")
public class PersonApartmentTestDataProvider implements ArgumentsProvider {

    String[] names = {
            "Miś Uszatek", "Gąska Balbinka", "Pies Pankracy", "Zając Poziomka", "Pepe LeSwąd"
    };
    boolean[] apartments = {true, false, true, false, true};
    WeWillStealYourFlatCheckResult[] eligibility = {
        ELIGIBLE, NOT_ELIGIBLE, ELIGIBLE, NOT_ELIGIBLE, ELIGIBLE
    };


    @Override
    public Stream<? extends Arguments> provideArguments(ExtensionContext extensionContext) {
        return IntStream
                .range(0, names.length)
                .mapToObj(this::buildPersonData)
                .map(Arguments::of);
    }

    private PersonTestData buildPersonData(int index) {
        return PersonTestData.of(buildPerson(index), eligibility[index]);
    }

    private Person buildPerson(int index) {
        var displayName = names[index];
        var nameSurname = displayName.split("\\s");
        return Person.builder()
                .displayName(displayName)
                .firstName(nameSurname[0])
                .lastName(nameSurname[1])
                .isMarried(false)
                .birthDate(LocalDate.now().minusYears(42))
                .housing(buildHousing(index))
                .build();
    }

    private Housing buildHousing(int index) {
        // This must always be worth millions
        var address = Address.builder()
                .withAddressLine1("Stumilowy Las")
                .withAddressLine2("")
                .withZipCode(TestZipCodes.PREMIUM_RATE_ZIP_CODE.getZipCode())
                .build();
        // but it not always be an apartment
        return Housing.builder()
                .area(1234d)
                .isApartment(apartments[index])
                .address(address)
                .build();
    }

}
