package pl.amu.edu.demo.logic;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ArgumentsSource;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import pl.amu.edu.demo.providers.AverageQuoteProvider;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.lenient;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
public class TestWeWillStealYourFlatProduct {

    @Mock
    AverageQuoteProvider provider;

    @BeforeEach
    public void setUp() {
        lenient().when(provider.getAverageQuotePerSquareMeter("98-765")).thenReturn(25000.0d);
    }

    @ParameterizedTest
    @ArgumentsSource(PersonAgeTestDataProvider.class)
    public void shouldAcceptOnlyAgeRange(PersonTestData testData) {
        commonTest(testData);
    }

    public void commonTest(PersonTestData testData) {
        // given
        var person = testData.person;
        var logic = new WeWillStealYourFlatProduct(provider);

        // when
        var result = logic.check(person);

        // then
        assertThat(result).isNotNull();
        assertThat(result).isEqualTo(testData.expectedResult);
    }

    @ParameterizedTest
    @ArgumentsSource(PersonApartmentTestDataProvider.class)
    public void shouldAcceptApartmentsOnly(PersonTestData testData) {
        commonTest(testData);
    }

    @ParameterizedTest
    @ArgumentsSource(PersonUnmarriedApartmentOwnersTestDataProvider.class)
    public void shouldDecideBasedOnNetWorthUnmarried(PersonTestData testData) {
        when(provider.getAverageQuotePerSquareMeter(TestZipCodes.AVERAGE_RATE_ZIP_CODE.getZipCode()))
                .thenReturn(TestZipCodes.AVERAGE_RATE_ZIP_CODE.getRatePerSquareMeters());
        commonTest(testData);
    }

    @ParameterizedTest
    @ArgumentsSource(PersonMarriedApartmentOwnersTestDataProvider.class)
    public void shouldDecideBasedOnNetWorthMarried(PersonTestData testData) {
        when(provider.getAverageQuotePerSquareMeter(TestZipCodes.CHEAPEST_RATE_ZIP_CODE.getZipCode()))
                .thenReturn(TestZipCodes.CHEAPEST_RATE_ZIP_CODE.getRatePerSquareMeters());
        commonTest(testData);
    }

}
