package pl.amu.edu.demo.maybe;

import pl.amu.edu.demo.data.Address;
import pl.amu.edu.demo.data.Housing;
import pl.amu.edu.demo.data.Person;

import java.time.LocalDate;
import java.util.Optional;

public class OptionalExample {

    public static void main(String[] args) {
        new OptionalExample().run();
    }

    private void run() {
        var person = Person.builder()
                .displayName("Gąska Balbinka").birthDate(LocalDate.ofYearDay(1959, 1)).build();
        var address = Address.builder().build();
        var underTheBridge = Housing.builder().area(0.2).address(address).build();
        var maybeAdult = Optional.ofNullable(person)
                .filter(Person::isAdult);
        var housing = maybeAdult
                .map(Person::getHousing)
                .orElse(underTheBridge);
        var zipCode = Optional.of(housing)
                .flatMap(Housing::zipCode)
                .orElseThrow();
        maybeAdult
                .map(Person::getBirthDate)
                .map(birthDate -> {
                    var name = person.displayName;
                    return String.format("%s: born %s lives at %s area.", name, birthDate, zipCode);
                })
                .ifPresent(System.out::println);
    }

}
