package pl.edu.amu.demo.life.dto;

import lombok.*;
import lombok.experimental.FieldDefaults;
import pl.edu.amu.demo.life.jpa.Person;
import pl.edu.amu.demo.life.jpa.Policy;
import pl.edu.amu.demo.life.jpa.PolicyStatus;

import java.math.BigDecimal;
import java.time.LocalDate;

@FieldDefaults(level = AccessLevel.PRIVATE)
@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode
@ToString
public class PolicyDTO {

    Long id;
    String number;
    BigDecimal insuredSum;
    LocalDate startDate;
    LocalDate endDate;
    PersonDTO insuredPerson;
    String status;
    LocalDate statusChangeDate;

    public Policy toPolicy() {
        var policyStatus = PolicyStatus.valueOf(status);
        return new Policy(
                id, number, insuredSum, startDate, endDate, insuredPerson.toPerson(), policyStatus, statusChangeDate
        );
    }

    public static PolicyDTO fromPolicy(Policy policy) {
        return new PolicyDTO(
                policy.getId(),
                policy.getNumber(),
                policy.getInsuredSum(),
                policy.getStartDate(),
                policy.getEndDate(),
                PersonDTO.fromPerson(policy.getInsuredPerson()),
                policy.getStatus().name(),
                policy.getStatusChangeDate()
        );
    }

}
