package pl.edu.amu.entity;

import jakarta.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;

import java.time.ZonedDateTime;
import java.util.Set;

@Entity
@NoArgsConstructor
@AllArgsConstructor
@Builder(builderClassName = "PersonBuilder", toBuilder = true)
@ToString
@FieldDefaults(level = AccessLevel.PUBLIC)
@Getter
public class Person {

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    @Column(name = "person_id")
    Long personId;

    @Column(name = "display_name", nullable = false)
    String displayName;

    @Column(nullable = false)
    @Enumerated(EnumType.STRING)
    PersonSex sex;

    @OneToMany(mappedBy = "person")
    Set<FamilyMember> familyMembers;

    @OneToMany(mappedBy = "person")
    Set<EventPerson> events;

    @Column(name = "creation_date", nullable = false)
    ZonedDateTime creationDate;

    @Column(name = "modification_date")
    ZonedDateTime modificationDate;

    Boolean alive;

    public Event findEventByType(EventType type) {
        return events.stream()
                .map(EventPerson::getEvent)
                .filter(ev -> ev.getType() == type)
                .findAny()
                .orElse(null);
    }

    public static final class PersonBuilder {
        ZonedDateTime creationDate = ZonedDateTime.now();
        ZonedDateTime modificationDate = ZonedDateTime.now();
    }

}
