package pl.edu.amu.repository;

import jakarta.transaction.Transactional;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import org.springframework.stereotype.Repository;
import pl.edu.amu.entity.AncestorFamily;
import pl.edu.amu.entity.FamilyMember;
import pl.edu.amu.entity.FamilyPersonKey;

import java.util.List;

@Repository
public interface FamilyMemberRepository extends CrudRepository<FamilyMember, FamilyPersonKey> {

    @Modifying
    @Transactional
    @Query(value = "INSERT INTO family_member (family_id, person_id, role) VALUES(?,?,?)", nativeQuery = true)
    void insert(Long familyId, Long personId, String role);


    @Query(value = """
                      SELECT fm.PERSON_ID, fm.role FROM FAMILY_MEMBER fm
                      JOIN FAMILY_MEMBER cfm ON(fm.FAMILY_ID = cfm.FAMILY_ID)
                      WHERE cfm.PERSON_ID = ?
                       AND cfm.ROLE = 'CHILD'
                       AND fm.ROLE in ('FATHER', 'MOTHER')""",
            nativeQuery = true
    )
    List<Long> findParentsForPersonId(Long personId);

    @Query(nativeQuery = true,
    value = """
    SELECT fm1.family_id AS familyId, p1.PERSON_ID AS motherId, p2.PERSON_ID AS fatherId
    FROM FAMILY_MEMBER fm1
    JOIN FAMILY_MEMBER fm2 ON (fm1.FAMILY_ID = fm2.FAMILY_ID)
    JOIN PERSON p1 ON(p1.PERSON_ID = fm1.PERSON_ID )
    JOIN PERSON p2 ON(p2.PERSON_ID = fm2.PERSON_ID )
    WHERE fm1.ROLE = 'MOTHER' AND fm2.ROLE = 'FATHER'
    AND fm1.PERSON_ID IN (SELECT ANCESTOR_ID FROM ANCESTOR WHERE person_id = ?1)
    AND fm2.PERSON_ID IN (SELECT ANCESTOR_ID FROM ANCESTOR WHERE person_id = ?1);
    """)
    List<AncestorFamily> findAncestorFamily(Long personId);


}
