package pl.edu.amu.web;

import lombok.AccessLevel;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import pl.edu.amu.service.DbService;
import pl.edu.amu.web.data.AncestorsData;

import java.util.List;

@FieldDefaults(makeFinal = true, level = AccessLevel.PRIVATE)
@Slf4j
@Controller
public class RootController {

    @NonNull DbService dbService;

    @Autowired
    public RootController(@NonNull DbService dbService) {
        this.dbService = dbService;
    }

    @RequestMapping(path = "/", method = RequestMethod.GET)
    public String getHomePage() {
        return "index";
    }

    @GetMapping(path = "/ancestors")
    public String getAncestorsForPerson(@RequestParam Long id,
                                        @RequestParam(defaultValue = "true") Boolean events,
                                        @RequestParam(defaultValue = "false") Boolean filter,
                                        Model model) {
        if (events) {
            List<AncestorsData> data = filter ?
                    dbService.findAncestorsData(id).stream()
                            .filter(AncestorsData::hasAtLeastOneFullEvent)
                            .toList() : dbService.findAncestorsData(id);
            model.addAttribute("ancestors", data);
            model.addAttribute("descendant", id);
            return "ancestors";
        }
        return "redirect:/";
    }

    @GetMapping(path = "/backup")
    public ResponseEntity<Resource> generateBackup() {
        var headers = new HttpHeaders();
        headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=ancestors.json");
        headers.add("Cache-Control", "no-cache, no-store, must-revalidate");
        headers.add("Pragma", "no-cache");
        headers.add("Expires", "0");
        var ancestors = dbService.getAllData();
        var resource = new ByteArrayResource(ancestors);
        return ResponseEntity.ok()
                .headers(headers)
                .contentLength(resource.contentLength())
                .body(resource);
    }

}
