package pl.edu.amu.web.data;

import lombok.Builder;
import lombok.Value;
import pl.edu.amu.entity.Event;

import static java.util.Objects.isNull;
import static java.util.Objects.nonNull;

@Value
@Builder
public class AncestorsData implements Comparable<AncestorsData> {

    Long personId;
    String displayName;
    String sex;
    Event birth;
    Event marriage;
    Event death;
    int level;
    int ancestorOffset;

    public boolean hasEvents() {
        return !(isNull(birth) && isNull(marriage) && isNull(death));
    }

    public boolean hasAtLeastOneFullEvent() {
        return (nonNull(birth) && birth.hasFullDate()) ||
                (nonNull(marriage) && marriage.hasFullDate()) ||
                (nonNull(death) && death.hasFullDate());
    }

    public boolean isEmpty() {
        return isNull(displayName) || displayName.isBlank();
    }

    public boolean nonEmpty() {
        return !isEmpty();
    }

    @Override
    public int compareTo(AncestorsData other) {
        if (isNull(other)) {
            return -1;
        }
        return Integer.compare(ancestorOffset, other.ancestorOffset);
    }

}
